@@ -40,7 +40,6 @@
 #include <QSslSocket>
 #include <QHostAddress>
 #include <QHostInfo>
-#include <QDBusConnection>
 #include <QDBusReply>
 #include <QDBusInterface>
 #include <QCache>
@@ -238,7 +237,7 @@ bool KProtocolManagerPrivate::shouldIgnoreProxyFor(const QUrl &url)
         }
 
         if (!address.isNull()) {
-            Q_FOREACH (const SubnetPair &subnet, noProxySubnets) {
+            for (const SubnetPair &subnet : qAsConst(noProxySubnets)) {
                 if (address.isInSubnet(subnet)) {
                     isMatch = true;
                     break;
@@ -506,7 +505,7 @@ QStringList KProtocolManagerPrivate::getSystemProxyFor(const QUrl &url)
     QNetworkProxyQuery query(url);
     const QList<QNetworkProxy> proxyList = QNetworkProxyFactory::systemProxyForQuery(query);
     proxies.reserve(proxyList.size());
-    Q_FOREACH (const QNetworkProxy &proxy, proxyList) {
+    for (const QNetworkProxy &proxy : proxyList) {
         QUrl url;
         const QNetworkProxy::ProxyType type = proxy.type();
         if (type == QNetworkProxy::NoProxy || type == QNetworkProxy::DefaultProxy) {
@@ -615,7 +614,7 @@ void KProtocolManager::badProxy(const QString &proxy)
     PRIVATE_DATA;
     QMutexLocker lock(&d->mutex);
     const QStringList keys(d->cachedProxyData.keys());
-    Q_FOREACH (const QString &key, keys) {
+    for (const QString &key : keys) {
         d->cachedProxyData[key]->removeAddress(proxy);
     }
 }
@@ -647,13 +646,6 @@ static void extractProxyCacheKeyFromUrl(const QUrl &u, QString *key)
 
 QString KProtocolManager::slaveProtocol(const QUrl &url, QStringList &proxyList)
 {
-#if 0
-    if (url.hasSubUrl()) { // We don't want the suburl's protocol
-        const QUrl::List list = QUrl::split(url);
-        return slaveProtocol(list.last(), proxyList);
-    }
-#endif
-
     proxyList.clear();
 
     // Do not perform a proxy lookup for any url classified as a ":local" url or
@@ -682,7 +674,7 @@ QString KProtocolManager::slaveProtocol(const QUrl &url, QStringList &proxyList)
     const int count = proxies.count();
 
     if (count > 0 && !(count == 1 && proxies.first() == QL1S("DIRECT"))) {
-        Q_FOREACH (const QString &proxy, proxies) {
+        for (const QString &proxy : proxies) {
             if (proxy == QL1S("DIRECT")) {
                 proxyList << proxy;
             } else {
@@ -700,7 +692,7 @@ QString KProtocolManager::slaveProtocol(const QUrl &url, QStringList &proxyList)
             && !protocol.startsWith(QLatin1String("http"))
             && !protocol.startsWith(QLatin1String("webdav"))
             && KProtocolInfo::isKnownProtocol(protocol)) {
-        Q_FOREACH (const QString &proxy, proxyList) {
+        for (const QString &proxy : qAsConst(proxyList)) {
             QUrl u(proxy);
             if (u.isValid() && KProtocolInfo::isKnownProtocol(u.scheme())) {
                 protocol = u.scheme();
@@ -947,7 +939,7 @@ QString KProtocolManager::userAgentForApplication(const QString &appName, const
         info += QL1S("; ") + machine;
     }
 
-    info += QL1S("; ") + extraInfo.join(QStringLiteral("; "));
+    info += QL1S("; ") + extraInfo.join(QLatin1String("; "));
 
     return (appName + QL1C('/') + appVersion + QStringLiteral(" (") + info + QL1C(')'));
 }
@@ -995,7 +987,7 @@ QString KProtocolManager::acceptLanguagesHeader()
     // Replace possible "C" in the language list with "en", unless "en" is
     // already pressent. This is to keep user's priorities in order.
     // If afterwards "en" is still not present, append it.
-    int idx = languageList.indexOf(QStringLiteral("C"));
+    int idx = languageList.indexOf(QLatin1String("C"));
     if (idx != -1) {
         if (languageList.contains(english)) {
             languageList.removeAt(idx);
@@ -1012,7 +1004,7 @@ QString KProtocolManager::acceptLanguagesHeader()
     KConfig acclangConf(QStringLiteral("accept-languages.codes"), KConfig::NoGlobals);
     KConfigGroup replacementCodes(&acclangConf, "ReplacementCodes");
     QStringList languageListFinal;
-    Q_FOREACH (const QString &lang, languageList) {
+    for (const QString &lang : qAsConst(languageList)) {
         const QStringList langs = replacementCodes.readEntry(lang, QStringList());
         if (langs.isEmpty()) {
             languageListFinal += lang;
@@ -1027,7 +1019,7 @@ QString KProtocolManager::acceptLanguagesHeader()
     // the value evenly
     int prio = 10;
     QString header;
-    Q_FOREACH (const QString &lang, languageListFinal) {
+    for (const QString &lang : qAsConst(languageListFinal)) {
         header += lang;
         if (prio < 10) {
             header += QL1S(";q=0.") + QString::number(prio);
@@ -1306,11 +1298,10 @@ QString KProtocolManager::protocolForArchiveMimetype(const QString &mimeType)
     QMutexLocker lock(&d->mutex);
     if (d->protocolForArchiveMimetypes.isEmpty()) {
         const QList<KProtocolInfoPrivate *> allProtocols = KProtocolInfoFactory::self()->allProtocols();
-        for (QList<KProtocolInfoPrivate *>::const_iterator it = allProtocols.begin();
-                it != allProtocols.end(); ++it) {
-            const QStringList archiveMimetypes = (*it)->m_archiveMimeTypes;
-            Q_FOREACH (const QString &mime, archiveMimetypes) {
-                d->protocolForArchiveMimetypes.insert(mime, (*it)->m_name);
+        for (KProtocolInfoPrivate *allProtocol : allProtocols) {
+            const QStringList archiveMimetypes = allProtocol->m_archiveMimeTypes;
+            for (const QString &mime : archiveMimetypes) {
+                d->protocolForArchiveMimetypes.insert(mime, allProtocol->m_name);
             }
         }
     }
