@@ -21,9 +21,6 @@
  *
  **/
 
-// TODO: remove me
-#undef QT_NO_CAST_FROM_ASCII
-
 #include "slavebase.h"
 
 #include <config-kiocore.h>
@@ -35,12 +32,14 @@
 #include <process.h>
 #endif
 
+#include <QtGlobal>
 #include <QFile>
 #include <QList>
 #include <QDateTime>
 #include <QElapsedTimer>
 #include <QCoreApplication>
 #include <QDataStream>
+#include <QMap>
 
 #include <kconfig.h>
 #include <kconfiggroup.h>
@@ -61,6 +60,12 @@
 #include <KAuth>
 #endif
 
+#if KIO_ASSERT_SLAVE_STATES
+#define KIO_STATE_ASSERT(cond, where, what) Q_ASSERT_X(cond, where, what)
+#else
+#define KIO_STATE_ASSERT(cond, where, what) do { if (!(cond)) qCWarning(KIO_CORE) << what; } while (false)
+#endif
+
 extern "C" {
     static void sigpipe_handler(int sig);
 }
@@ -113,19 +118,21 @@ public:
     bool inOpenLoop: 1;
     bool exit_loop: 1;
     MetaData configData;
-    KConfig *config;
-    KConfigGroup *configGroup;
+    KConfig *config = nullptr;
+    KConfigGroup *configGroup = nullptr;
+    QMap<QString, QVariant> mapConfig;
     QUrl onHoldUrl;
 
     QElapsedTimer lastTimeout;
     QElapsedTimer nextTimeout;
     qint64 nextTimeoutMsecs;
     KIO::filesize_t totalSize;
-    KRemoteEncoding *remotefile;
+    KRemoteEncoding *remotefile = nullptr;
     enum { Idle, InsideMethod, FinishedCalled, ErrorCalled } m_state;
+    bool m_finalityCommand = true; // whether finished() or error() may/must be called
     QByteArray timeoutData;
 
-    KPasswdServerClient *m_passwdServerClient;
+    KPasswdServerClient *m_passwdServerClient = nullptr;
     bool m_rootEntryListed = false;
 
     bool m_confirmationAsked;
@@ -170,33 +177,54 @@ public:
     // Reconstructs configGroup from configData and mIncomingMetaData
     void rebuildConfig()
     {
-        configGroup->deleteGroup(KConfigGroup::WriteConfigFlags());
+        mapConfig.clear();
 
         // mIncomingMetaData cascades over config, so we write config first,
         // to let it be overwritten
         MetaData::ConstIterator end = configData.constEnd();
         for (MetaData::ConstIterator it = configData.constBegin(); it != end; ++it) {
-            configGroup->writeEntry(it.key(), it->toUtf8(), KConfigGroup::WriteConfigFlags());
+            mapConfig.insert(it.key(), it->toUtf8());
         }
 
         end = q->mIncomingMetaData.constEnd();
         for (MetaData::ConstIterator it = q->mIncomingMetaData.constBegin(); it != end; ++it) {
-            configGroup->writeEntry(it.key(), it->toUtf8(), KConfigGroup::WriteConfigFlags());
+            mapConfig.insert(it.key(), it->toUtf8());
         }
+
+        delete configGroup;
+        configGroup = nullptr;
+        delete config;
+        config = nullptr;
+    }
+
+    bool finalState() const
+    {
+        return ((m_state == FinishedCalled) || (m_state == ErrorCalled));
     }
 
     void verifyState(const char *cmdName)
     {
-        if ((m_state != FinishedCalled) && (m_state != ErrorCalled)) {
-            qCWarning(KIO_CORE) << cmdName << "did not call finished() or error()! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
+        KIO_STATE_ASSERT(finalState(),
+                         Q_FUNC_INFO,
+                         qUtf8Printable(QStringLiteral("%1 did not call finished() or error()! Please fix the %2 KIO slave")
+                                        .arg(QLatin1String(cmdName))
+                                        .arg(QCoreApplication::applicationName())));
+        // Force the command into finished state. We'll not reach this for Debug builds
+        // that fail the assertion. For Release builds we'll have made sure that the
+        // command is actually finished after the verification regardless of what
+        // the slave did.
+        if (!finalState()) {
+            q->finished();
         }
     }
 
     void verifyErrorFinishedNotCalled(const char *cmdName)
     {
-        if (m_state == FinishedCalled || m_state == ErrorCalled) {
-            qCWarning(KIO_CORE) << cmdName << "called finished() or error(), but it's not supposed to! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
-        }
+        KIO_STATE_ASSERT(!finalState(),
+                         Q_FUNC_INFO,
+                         qUtf8Printable(QStringLiteral("%1 called finished() or error(), but it's not supposed to! Please fix the %2 KIO slave")
+                                        .arg(QLatin1String(cmdName))
+                                        .arg(QCoreApplication::applicationName())));
     }
 
     KPasswdServerClient *passwdServerClient()
@@ -271,9 +299,7 @@ SlaveBase::SlaveBase(const QByteArray &protocol,
     d->slaveid = QString::fromUtf8(protocol) + QString::number(getpid());
     d->resume = false;
     d->needSendCanResume = false;
-    d->config = new KConfig(QString(), KConfig::SimpleConfig);
-    // The KConfigGroup needs the KConfig to exist during its whole lifetime.
-    d->configGroup = new KConfigGroup(d->config, QString());
+    d->mapConfig = QMap<QString, QVariant>();
     d->onHold = false;
     d->wasKilled = false;
 //    d->processed_size = 0;
@@ -404,8 +430,40 @@ bool SlaveBase::hasMetaData(const QString &key) const
     return false;
 }
 
+QMap<QString, QVariant> SlaveBase::mapConfig() const
+{
+    return d->mapConfig;
+}
+
+bool SlaveBase::configValue(const QString &key, bool defaultValue) const
+{
+    return d->mapConfig.value(key, defaultValue).toBool();
+}
+
+int SlaveBase::configValue(const QString &key, int defaultValue) const
+{
+    return d->mapConfig.value(key, defaultValue).toInt();
+}
+
+QString SlaveBase::configValue(const QString &key, const QString &defaultValue) const
+{
+    return d->mapConfig.value(key, defaultValue).toString();
+}
+
 KConfigGroup *SlaveBase::config()
 {
+    if (!d->config) {
+        d->config = new KConfig(QString(), KConfig::SimpleConfig);
+
+        d->configGroup = new KConfigGroup(d->config, QString());
+
+        auto end = d->mapConfig.cend();
+        for (auto it = d->mapConfig.cbegin(); it != end; ++it)
+        {
+            d->configGroup->writeEntry(it.key(), it->toString().toUtf8(), KConfigGroup::WriteConfigFlags());
+        }
+    }
+
     return d->configGroup;
 }
 
@@ -458,11 +516,22 @@ void SlaveBase::opened()
 
 void SlaveBase::error(int _errid, const QString &_text)
 {
+    KIO_STATE_ASSERT(d->m_finalityCommand,
+                     Q_FUNC_INFO,
+                     qUtf8Printable(QStringLiteral("error() was called, but it's not supposed to! Please fix the %1 KIO slave")
+                                    .arg(QCoreApplication::applicationName())));
+
     if (d->m_state == d->ErrorCalled) {
-        qCWarning(KIO_CORE) << "error() called twice! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
+        KIO_STATE_ASSERT(false,
+                         Q_FUNC_INFO,
+                         qUtf8Printable(QStringLiteral("error() called twice! Please fix the %1 KIO slave")
+                                        .arg(QCoreApplication::applicationName())));
         return;
     } else if (d->m_state == d->FinishedCalled) {
-        qCWarning(KIO_CORE) << "error() called after finished()! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
+        KIO_STATE_ASSERT(false,
+                         Q_FUNC_INFO,
+                         qUtf8Printable(QStringLiteral("error() called after finished()! Please fix the %1 KIO slave")
+                                        .arg(QCoreApplication::applicationName())));
         return;
     }
 
@@ -502,11 +571,22 @@ void SlaveBase::finished()
         d->pendingListEntries.clear();
     }
 
+    KIO_STATE_ASSERT(d->m_finalityCommand,
+                     Q_FUNC_INFO,
+                     qUtf8Printable(QStringLiteral("finished() was called, but it's not supposed to! Please fix the %2 KIO slave")
+                                    .arg(QCoreApplication::applicationName())));
+
     if (d->m_state == d->FinishedCalled) {
-        qCWarning(KIO_CORE) << "finished() called twice! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
+        KIO_STATE_ASSERT(false,
+                         Q_FUNC_INFO,
+                         qUtf8Printable(QStringLiteral("finished() called twice! Please fix the %1 KIO slave")
+                                        .arg(QCoreApplication::applicationName())));
         return;
     } else if (d->m_state == d->ErrorCalled) {
-        qCWarning(KIO_CORE) << "finished() called after error()! Please fix the" << QCoreApplication::applicationName() << "KIO slave";
+        KIO_STATE_ASSERT(false,
+                         Q_FUNC_INFO,
+                         qUtf8Printable(QStringLiteral("finished() called after error()! Please fix the %1 KIO slave")
+                                        .arg(QCoreApplication::applicationName())));
         return;
     }
 
@@ -679,7 +759,6 @@ void SlaveBase::infoMessage(const QString &_msg)
     send(INF_INFOMESSAGE, data);
 }
 
-#ifndef KIOCORE_NO_DEPRECATED
 bool SlaveBase::requestNetwork(const QString &host)
 {
     KIO_DATA << host << d->slaveid;
@@ -700,7 +779,6 @@ void SlaveBase::dropNetwork(const QString &host)
     KIO_DATA << host << d->slaveid;
     send(MSG_NET_DROP, data);
 }
-#endif
 
 void SlaveBase::statEntry(const UDSEntry &entry)
 {
@@ -708,13 +786,12 @@ void SlaveBase::statEntry(const UDSEntry &entry)
     send(MSG_STAT_ENTRY, data);
 }
 
-#ifndef KIOCORE_NO_DEPRECATED
 void SlaveBase::listEntry(const UDSEntry &entry, bool _ready)
 {
     if (_ready) {
         // #366795: many slaves don't create an entry for ".", so we keep track if they do
         // and we provide a fallback in finished() otherwise.
-        if (entry.stringValue(KIO::UDSEntry::UDS_NAME) == QLatin1String(".")) {
+        if (entry.stringValue(KIO::UDSEntry::UDS_NAME) == QLatin1Char('.')) {
             d->m_rootEntryListed = true;
         }
         listEntries(d->pendingListEntries);
@@ -723,13 +800,12 @@ void SlaveBase::listEntry(const UDSEntry &entry, bool _ready)
         listEntry(entry);
     }
 }
-#endif
 
 void SlaveBase::listEntry(const UDSEntry &entry)
 {
     // #366795: many slaves don't create an entry for ".", so we keep track if they do
     // and we provide a fallback in finished() otherwise.
-    if (entry.stringValue(KIO::UDSEntry::UDS_NAME) == QLatin1String(".")) {
+    if (entry.stringValue(KIO::UDSEntry::UDS_NAME) == QLatin1Char('.')) {
         d->m_rootEntryListed = true;
     }
 
@@ -824,49 +900,49 @@ KIOCORE_EXPORT QString KIO::unsupportedActionErrorString(const QString &protocol
 
 void SlaveBase::openConnection()
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_CONNECT));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_CONNECT));
 }
 void SlaveBase::closeConnection()
 { } // No response!
 void SlaveBase::stat(QUrl const &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_STAT));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_STAT));
 }
 void SlaveBase::put(QUrl const &, int, JobFlags)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_PUT));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_PUT));
 }
 void SlaveBase::special(const QByteArray &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SPECIAL));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_SPECIAL));
 }
 void SlaveBase::listDir(QUrl const &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_LISTDIR));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_LISTDIR));
 }
 void SlaveBase::get(QUrl const &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_GET));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_GET));
 }
 void SlaveBase::open(QUrl const &, QIODevice::OpenMode)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_OPEN));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_OPEN));
 }
 void SlaveBase::read(KIO::filesize_t)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_READ));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_READ));
 }
 void SlaveBase::write(const QByteArray &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_WRITE));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_WRITE));
 }
 void SlaveBase::seek(KIO::filesize_t)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SEEK));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_SEEK));
 }
 void SlaveBase::close()
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_CLOSE));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_CLOSE));
 }
 void SlaveBase::mimetype(QUrl const &url)
 {
@@ -874,47 +950,47 @@ void SlaveBase::mimetype(QUrl const &url)
 }
 void SlaveBase::rename(QUrl const &, QUrl const &, JobFlags)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_RENAME));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_RENAME));
 }
 void SlaveBase::symlink(QString const &, QUrl const &, JobFlags)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SYMLINK));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_SYMLINK));
 }
 void SlaveBase::copy(QUrl const &, QUrl const &, int, JobFlags)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_COPY));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_COPY));
 }
 void SlaveBase::del(QUrl const &, bool)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_DEL));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_DEL));
 }
 void SlaveBase::setLinkDest(const QUrl &, const QString &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SETLINKDEST));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_SETLINKDEST));
 }
 void SlaveBase::mkdir(QUrl const &, int)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_MKDIR));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_MKDIR));
 }
 void SlaveBase::chmod(QUrl const &, int)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_CHMOD));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_CHMOD));
 }
 void SlaveBase::setModificationTime(QUrl const &, const QDateTime &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SETMODIFICATIONTIME));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_SETMODIFICATIONTIME));
 }
 void SlaveBase::chown(QUrl const &, const QString &, const QString &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_CHOWN));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_CHOWN));
 }
 void SlaveBase::setSubUrl(QUrl const &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SUBURL));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_SUBURL));
 }
 void SlaveBase::multiGet(const QByteArray &)
 {
-    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_MULTI_GET));
+    error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_MULTI_GET));
 }
 
 void SlaveBase::slave_status()
@@ -1062,6 +1138,8 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
     QUrl url;
     int i;
 
+    d->m_finalityCommand = true; // default
+
     switch (command) {
     case CMD_HOST: {
         QString passwd;
@@ -1069,8 +1147,8 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
         quint16 port;
         stream >> host >> port >> user >> passwd;
         d->m_state = d->InsideMethod;
+        d->m_finalityCommand = false;
         setHost(host, port, user, passwd);
-        d->verifyErrorFinishedNotCalled("setHost()");
         d->m_state = d->Idle;
     } break;
     case CMD_CONNECT: {
@@ -1081,9 +1159,9 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
     } break;
     case CMD_SLAVE_STATUS: {
         d->m_state = d->InsideMethod;
+        d->m_finalityCommand = false;
         slave_status();
         // TODO verify that the slave has called slaveStatus()?
-        d->verifyErrorFinishedNotCalled("slave_status()");
         d->m_state = d->Idle;
     } break;
     case CMD_SLAVE_CONNECT: {
@@ -1110,8 +1188,8 @@ void SlaveBase::dispatch(int command, const QByteArray &data)
     } break;
     case CMD_REPARSECONFIGURATION: {
         d->m_state = d->InsideMethod;
+        d->m_finalityCommand = false;
         reparseConfiguration();
-        d->verifyErrorFinishedNotCalled("reparseConfiguration()");
         d->m_state = d->Idle;
     } break;
     case CMD_CONFIG: {
@@ -1419,7 +1497,7 @@ void SlaveBase::virtual_hook(int id, void *data)
 
     switch(id) {
     case GetFileSystemFreeSpace: {
-        error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_FILESYSTEMFREESPACE));
+        error(ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(protocolName(), CMD_FILESYSTEMFREESPACE));
     } break;
     }
 }
