@@ -25,8 +25,8 @@
 #include <kio/authinfo.h>
 #include "job_base.h" // for KIO::JobFlags
 
-#include <QtCore/QByteArray>
-#include <QtNetwork/QHostInfo>
+#include <QByteArray>
+#include <QHostInfo>
 
 class KConfigGroup;
 class KRemoteEncoding;
@@ -39,6 +39,8 @@ class Connection;
 class SlaveBasePrivate;
 
 /**
+ * @class KIO::SlaveBase slavebase.h <KIO/SlaveBase>
+ *
  * There are two classes that specifies the protocol between application (job)
  * and kioslave. SlaveInterface is the class to use on the application end,
  * SlaveBase is the one to use on the slave end.
@@ -46,6 +48,15 @@ class SlaveBasePrivate;
  * Slave implementations should simply inherit SlaveBase
  *
  * A call to foo() results in a call to slotFoo() on the other end.
+ *
+ * Note that a kioslave doesn't have a Qt event loop. When idle, it's waiting for a command
+ * on the socket that connects it to the application. So don't expect a kioslave to react
+ * to D-Bus signals for instance. KIOSlaves are short-lived anyway, so any kind of watching
+ * or listening for notifications should be done elsewhere, for instance in a kded module
+ * (see kio_desktop's desktopnotifier.cpp for an example).
+ *
+ * If a kioslave needs a Qt event loop within the implementation of one method, e.g. to
+ * wait for an asynchronous operation to finish, that is possible, using QEventLoop.
  */
 class KIOCORE_EXPORT SlaveBase
 {
@@ -57,7 +68,7 @@ public:
      * @internal
      * Terminate the slave by calling the destructor and then ::exit()
      */
-    void exit();
+    Q_NORETURN void exit();
 
     /**
      * @internal
@@ -85,8 +96,8 @@ public:
     void dataReq();
 
     /**
-     * open succedes
-     * @see open
+     * open succeeds
+     * @see open()
      */
     void opened();
 
@@ -100,7 +111,7 @@ public:
      * of the error message.
      *
      * For all other error codes, _text should match the corresponding
-     * error code. Usually,, _text is a file or host name, or the error which
+     * error code. Usually, _text is a file or host name, or the error which
      * was passed from the server.<br>
      * For example, for KIO::ERR_DOES_NOT_EXIST, _text may only
      * be the file or folder which does not exist, nothing else. Otherwise,
@@ -170,8 +181,8 @@ public:
     bool canResume(KIO::filesize_t offset);
 
     /**
-     * Call this at the beginning of get(), if the "resume" metadata was set
-     * and resuming is implemented by this protocol.
+     * Call this at the beginning of get(), if the "range-start" metadata was set
+     * and returning byte ranges is implemented by this protocol.
      */
     void canResume();
 
@@ -290,8 +301,7 @@ public:
      *                  The default is i18n("&No").
      * Note: for ContinueCancel, buttonYes is the continue button and buttonNo is unused.
      *       and for Information, none is used.
-     * @param dontAskAgain A checkbox is added with which further confirmation can be turned off.
-     *        If the checkbox was ticked @p dontAskAgain will be set to true, otherwise false.
+     * @param dontAskAgainName the name used to store result from 'Do not ask again' checkbox.
      * @return a button code, as defined in ButtonCode, or 0 on communication error.
      */
     int messageBox(const QString &text, MessageBoxType type,
@@ -323,14 +333,49 @@ public:
      */
     MetaData allMetaData() const;
 
+    /**
+     * Returns a map to query config/meta-data information from.
+     *
+     * The application provides the slave with all configuration information
+     * relevant for the current protocol and host.
+     *
+     * Use configValue() as shortcut.
+     * @since 5.64
+     */
+    QMap<QString, QVariant> mapConfig() const;
+
+    /**
+     * Returns a bool from the config/meta-data information.
+     * @since 5.64
+     */
+    bool configValue(const QString &key, bool defaultValue) const;
+
+    /**
+     * Returns an int from the config/meta-data information.
+     * @since 5.64
+     */
+    int configValue(const QString &key, int defaultValue) const;
+
+    /**
+     * Returns a QString from the config/meta-data information.
+     * @since 5.64
+     */
+    QString configValue(const QString &key, const QString &defaultValue = QString()) const;
+
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 64)
     /**
      * Returns a configuration object to query config/meta-data information
      * from.
      *
      * The application provides the slave with all configuration information
      * relevant for the current protocol and host.
+     *
+     * @deprecated since 5.64 use mapConfig instead
      */
+    KIOCORE_DEPRECATED_VERSION(5, 64, "Use SlaveBase::mapConfig()")
     KConfigGroup *config();
+    // KF6: perhaps rename mapConfig() to config() when removing this
+#endif
 
     /**
      * Returns an object that can translate remote filenames into proper
@@ -345,10 +390,7 @@ public:
 
     /**
      * Set the host
-     * @param host
-     * @param port
-     * @param user
-     * @param pass
+     *
      * Called directly by createSlave, this is why there is no equivalent in
      * SlaveInterface, unlike the other methods.
      *
@@ -411,9 +453,28 @@ public:
      */
     virtual void open(const QUrl &url, QIODevice::OpenMode mode);
 
+    /**
+     * read.
+     * @param size the requested amount of data to read
+     * @see KIO::FileJob::read()
+     */
     virtual void read(KIO::filesize_t size);
+    /**
+     * write.
+     * @param data the data to write
+     * @see KIO::FileJob::write()
+     */
     virtual void write(const QByteArray &data);
+    /**
+     * seek.
+     * @param offset the requested amount of data to read
+     * @see KIO::FileJob::read()
+     */
     virtual void seek(KIO::filesize_t offset);
+    /**
+     * close.
+     * @see KIO::FileJob::close()
+     */
     virtual void close();
 
     /**
@@ -421,7 +482,7 @@ public:
      *
      * @param url where to write the file
      * @param permissions may be -1. In this case no special permission mode is set.
-     * @param flags: We support Overwrite here. Hopefully, we're going to
+     * @param flags We support Overwrite here. Hopefully, we're going to
      * support Resume in the future, too.
      * If the file indeed already exists, the slave should NOT apply the
      * permissions change to it.
@@ -480,7 +541,7 @@ public:
      * @param url path to the directory to create
      * @param permissions the permissions to set after creating the directory
      * (-1 if no permissions to be set)
-     * The slave emits ERR_COULD_NOT_MKDIR if failure.
+     * The slave emits ERR_CANNOT_MKDIR if failure.
      */
     virtual void mkdir(const QUrl &url, int permissions);
 
@@ -508,7 +569,7 @@ public:
      *
      * @param src where to move the file from
      * @param dest where to move the file to
-     * @param flags: We support Overwrite here
+     * @param flags We support Overwrite here
      */
     virtual void rename(const QUrl &src, const QUrl &dest, JobFlags flags);
 
@@ -517,7 +578,7 @@ public:
      * may be a relative or an absolute path.
      * @param target The string that will become the "target" of the link (can be relative)
      * @param dest The symlink to create.
-     * @param flags: We support Overwrite here
+     * @param flags We support Overwrite here
      */
     virtual void symlink(const QString &target, const QUrl &dest, JobFlags flags);
 
@@ -562,7 +623,7 @@ public:
      * @param src where to copy the file from (decoded)
      * @param dest where to copy the file to (decoded)
      * @param permissions may be -1. In this case no special permission mode is set.
-     * @param flags: We support Overwrite here
+     * @param flags We support Overwrite here
      *
      * Don't forget to set the modification time of @p dest to be the modification time of @p src.
      */
@@ -598,7 +659,7 @@ public:
     virtual void special(const QByteArray &data);
 
     /**
-     * Used for multiple get. Currently only used foir HTTP pielining
+     * Used for multiple get. Currently only used for HTTP pipelining
      * support.
      *
      * @param data packed data; Contains number of URLs to fetch, and for
@@ -679,6 +740,7 @@ public:
      **/
     int readData(QByteArray &buffer);
 
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
      * It collects entries and emits them via listEntries
      * when enough of them are there or a certain time
@@ -694,8 +756,8 @@ public:
      * You should make sure that finished() is called when the entry
      * listing is completed and simply remove the call to listEntry(entry, true).
      */
-#ifndef KDE_NO_DEPRECATED
-    KIOCORE_DEPRECATED void listEntry(const UDSEntry &_entry, bool ready);
+    KIOCORE_DEPRECATED_VERSION(5, 0, "See API docs")
+    void listEntry(const UDSEntry &_entry, bool ready);
 #endif
 
     /**
@@ -726,12 +788,12 @@ public:
      *
      * \code
      * KIO::AuthInfo authInfo;
-     * if ( openPasswordDialog( authInfo ) )
-     * {
-     *    qDebug() << QLatin1String("User: ")
-     *              << authInfo.username << endl;
-     *    qDebug() << QLatin1String("Password: ")
-     *              << QLatin1String("Not displayed here!") << endl;
+     * int errorCode = openPasswordDialogV2(authInfo);
+     * if (!errorCode) {
+     *    qDebug() << QLatin1String("User: ") << authInfo.username;
+     *    qDebug() << QLatin1String("Password: not displayed here!");
+     * } else {
+     *    error(errorCode, QString());
      * }
      * \endcode
      *
@@ -740,16 +802,11 @@ public:
      *
      * \code
      * KIO::AuthInfo authInfo;
-     * authInfo.caption= "Acme Password Dialog";
-     * authInfo.username= "Wile E. Coyote";
-     * QString errorMsg = "You entered an incorrect password.";
-     * if ( openPasswordDialog( authInfo, errorMsg ) )
-     * {
-     *    qDebug() << QLatin1String("User: ")
-     *              << authInfo.username << endl;
-     *    qDebug() << QLatin1String("Password: ")
-     *              << QLatin1String("Not displayed here!") << endl;
-     * }
+     * authInfo.caption = i18n("Acme Password Dialog");
+     * authInfo.username = "Wile E. Coyote";
+     * QString errorMsg = i18n("You entered an incorrect password.");
+     * int errorCode = openPasswordDialogV2(authInfo, errorMsg);
+     * [...]
      * \endcode
      *
      * \note You should consider using checkCachedAuthentication() to
@@ -757,7 +814,7 @@ public:
      * this function.
      *
      * \note A call to this function can fail and return @p false,
-     * if the UIServer could not be started for whatever reason.
+     * if the password server could not be started for whatever reason.
      *
      * \note This function does not store the password information
      * automatically (and has not since kdelibs 4.7). If you want to
@@ -767,9 +824,22 @@ public:
      * @see checkCachedAuthentication
      * @param info  See AuthInfo.
      * @param errorMsg Error message to show
-     * @return      @p true if user clicks on "OK", @p false otherwsie.
+     * @return a KIO error code: NoError (0), KIO::USER_CANCELED, or other error codes.
      */
+    int openPasswordDialogV2(KIO::AuthInfo &info, const QString &errorMsg = QString());
+
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 24)
+    /**
+     * @deprecated since KF 5.24, use openPasswordDialogV2.
+     * The return value works differently:
+     *  instead of
+     *        if (!openPasswordDialog()) { error(USER_CANCELED); }
+     *  store and pass the return value to error(), when NOT zero,
+     *  as shown documentation for openPasswordDialogV2().
+     */
+    KIOCORE_DEPRECATED_VERSION(5, 24, "Use SlaveBase::openPasswordDialogV2(...)")
     bool openPasswordDialog(KIO::AuthInfo &info, const QString &errorMsg = QString());
+#endif
 
     /**
      * Checks for cached authentication based on parameters
@@ -790,7 +860,7 @@ public:
      * info.verifyPath = true;
      * if ( !checkCachedAuthentication( info ) )
      * {
-     *    if ( !openPasswordDialog(info) )
+     *    int errorCode = openPasswordDialogV2(info);
      *     ....
      * }
      * \endcode
@@ -803,7 +873,7 @@ public:
     /**
      * Caches @p info in a persistent storage like KWallet.
      *
-     * Note that calling openPasswordDialog does not store passwords
+     * Note that calling openPasswordDialogV2 does not store passwords
      * automatically for you (and has not since kdelibs 4.7).
      *
      * Here is a simple example of how to use cacheAuthentication:
@@ -814,7 +884,8 @@ public:
      * info.username = "somename";
      * info.verifyPath = true;
      * if ( !checkCachedAuthentication( info ) ) {
-     *    if ( openPasswordDialog(info) ) {
+     *    int errorCode = openPasswordDialogV2(info);
+     *    if (!errorCode) {
      *        if (info.keepPassword)  {  // user asked password be save/remembered
      *             cacheAuthentication(info);
      *        }
@@ -827,10 +898,8 @@ public:
      */
     bool cacheAuthentication(const AuthInfo &info);
 
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
-     * @deprecated for a very very long time, not implemented anymore
-     * Probably dates back to model dialup times.
-     *
      * Used by the slave to check if it can connect
      * to a given host. This should be called where the slave is ready
      * to do a ::connect() on a socket. For each call to
@@ -842,19 +911,21 @@ public:
      *
      * @param host tells the netmgr the host the slave wants to connect
      *             to. As this could also be a proxy, we can't just take
-     *             the host currenctly connected to (but that's the default
+     *             the host currently connected to (but that's the default
      *             value)
      *
      * @return true in theory, the host is reachable
      *         false the system is offline and the host is in a remote network.
+     *
+     * @deprecated Since 5.0, for a very very long time, not implemented anymore
+     * Probably dates back to model dialup times.
      */
-#ifndef KDE_NO_DEPRECATED
+    KIOCORE_DEPRECATED_VERSION(5, 0, "Not implemented & used")
     bool requestNetwork(const QString &host = QString());
 #endif
 
+#if KIOCORE_ENABLE_DEPRECATED_SINCE(5, 0)
     /**
-     * @deprecated for a very very long time, not implemented anymore
-     * Probably dates back to model dialup times.
      *
      * Used by the slave to withdraw a connection requested by
      * requestNetwork. This function cancels the last call to
@@ -867,8 +938,11 @@ public:
      * @param host the host passed to requestNetwork
      *
      * A slave should call this function every time it disconnect from a host.
+     * *
+     * @deprecated Since 5.0, for a very very long time, not implemented anymore
+     * Probably dates back to model dialup times.
      * */
-#ifndef KDE_NO_DEPRECATED
+    KIOCORE_DEPRECATED_VERSION(5, 0, "Not implemented & used")
     void dropNetwork(const QString &host = QString());
 #endif
 
@@ -876,7 +950,7 @@ public:
      * Wait for an answer to our request, until we get @p expected1 or @p expected2
      * @return the result from readData, as well as the cmd in *pCmd if set, and the data in @p data
      */
-    int waitForAnswer(int expected1, int expected2, QByteArray &data, int *pCmd = 0);
+    int waitForAnswer(int expected1, int expected2, QByteArray &data, int *pCmd = nullptr);
 
     /**
      * Internal function to transmit meta data to the application.
@@ -922,6 +996,23 @@ public:
     */
     int waitForHostInfo(QHostInfo &info);
 
+    /**
+     * Checks with job if privilege operation is allowed.
+     * @return privilege operation status.
+     * @see PrivilegeOperationStatus
+     * @since 5.43
+     */
+    PrivilegeOperationStatus requestPrivilegeOperation();
+
+    /**
+      * Adds @p action to the list of PolicyKit actions which the
+      * slave is authorized to perform.
+      *
+      * @param action the PolicyKit action
+      * @since 5.45
+      */
+    void addTemporaryAuthorization(const QString &action);
+
 protected:
     /**
      * Name of the protocol supported by this slave
@@ -932,11 +1023,18 @@ protected:
     MetaData mIncomingMetaData;
 
     enum VirtualFunctionId {
-        AppConnectionMade = 0
+        AppConnectionMade = 0,
+        GetFileSystemFreeSpace = 1   // KF6 TODO: Turn into a virtual method
     };
     virtual void virtual_hook(int id, void *data);
 
 private:
+    // Convenience function converting mProtocol to QString as unsupportedActionErrorString(), which
+    // is used in many places in the code, takes a QString parameter
+    inline const QString protocolName() const {
+        return QString::fromLatin1(mProtocol);
+    }
+
     // This helps catching missing tr()/i18n() calls in error().
     void error(int _errid, const QByteArray &_text);
     void send(int cmd, const QByteArray &arr = QByteArray());
