@@ -45,6 +45,7 @@
 #include <QTimer>
 #include <QDebug>
 #include <QMimeDatabase>
+#include <QStack>
 
 #include <kdirlister.h>
 #include <kfileitemdelegate.h>
@@ -69,16 +70,6 @@ template class QHash<QString, KFileItem>;
 // sorting mode.
 static const int QDirSortMask = QDir::SortByMask | QDir::Type;
 
-
-void KDirOperator::keyPressEvent(QKeyEvent *e)
-{
-    if (!(e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter)) {
-        QWidget::keyPressEvent(e);
-    } else {
-        emit keyEnterReturnPressed();
-    }
-}
-
 class Q_DECL_HIDDEN KDirOperator::Private
 {
 public:
@@ -196,6 +187,8 @@ public:
 
     bool dirHighlighting;
     bool onlyDoubleClickSelectsFiles;
+    bool followNewDirectories;
+    bool followSelectedDirectories;
     QString lastURL; // used for highlighting a directory on cdUp
     QTimer *progressDelayTimer;
     int dropOptions;
@@ -236,6 +229,8 @@ KDirOperator::Private::Private(KDirOperator *_parent) :
     previewWidth(0),
     dirHighlighting(false),
     onlyDoubleClickSelectsFiles(!qApp->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick)),
+    followNewDirectories(true),
+    followSelectedDirectories(true),
     progressDelayTimer(nullptr),
     dropOptions(0),
     actionMenu(nullptr),
@@ -1286,6 +1281,8 @@ void KDirOperator::activatedMenu(const KFileItem &item, const QPoint &pos)
     d->newFileMenu->setViewShowsHiddenFiles(showHiddenFiles());
     d->newFileMenu->checkUpToDate();
 
+    d->actionCollection->action(QStringLiteral("new"))->setEnabled(item.isDir());
+
     emit contextMenuAboutToShow(item, d->actionMenu->menu());
 
     d->actionMenu->menu()->exec(pos);
@@ -1358,7 +1355,7 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
     case QEvent::Wheel: {
         QWheelEvent *evt = static_cast<QWheelEvent *>(event);
         if (evt->modifiers() & Qt::ControlModifier) {
-            if (evt->delta() > 0) {
+            if (evt->angleDelta().y() > 0) {
                 setIconsZoom(d->iconsZoom + 10);
             } else {
                 setIconsZoom(d->iconsZoom - 10);
@@ -1383,7 +1380,7 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
 
             if (mimeFilters.size() > 1) {
                 mimeFilterPass = false;
-                const QUrl url = urls.constFirst();
+                const QUrl &url = urls.constFirst();
 
                 QMimeDatabase mimeDataBase;
                 QMimeType fileMimeType = mimeDataBase.mimeTypeForUrl(url);
@@ -1407,7 +1404,7 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
         QDropEvent *evt = static_cast<QDropEvent *>(event);
         const QList<QUrl> urls = KUrlMimeData::urlsFromMimeData(evt->mimeData(), KUrlMimeData::DecodeOptions::PreferLocalUrls);
 
-        const QUrl url = urls.constFirst();
+        const QUrl &url = urls.constFirst();
 
         // stat the url to get details
         KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
@@ -1438,6 +1435,19 @@ bool KDirOperator::eventFilter(QObject *watched, QEvent *event)
         evt->accept();
         return true;
     }
+    case QEvent::KeyPress: {
+        QKeyEvent *evt = static_cast<QKeyEvent *>(event);
+        if (evt->key() == Qt::Key_Return || evt->key() == Qt::Key_Enter) {
+            // when no elements are selected and Return/Enter is pressed
+            // emit keyEnterReturnPressed
+            // let activated event be emitted by subsequent QAbstractItemView::keyPress otherwise
+            if (!d->itemView->currentIndex().isValid()) {
+                emit keyEnterReturnPressed();
+                evt->accept();
+                return true;
+            }
+        }
+    }
     default:
         break;
     }
@@ -1449,7 +1459,7 @@ bool KDirOperator::Private::checkPreviewInternal() const
 {
     const QStringList supported = KIO::PreviewJob::supportedMimeTypes();
     // no preview support for directories?
-    if (parent->dirOnlyMode() && supported.indexOf(QStringLiteral("inode/directory")) == -1) {
+    if (parent->dirOnlyMode() && supported.indexOf(QLatin1String("inode/directory")) == -1) {
         return false;
     }
 
@@ -1926,7 +1936,8 @@ void KDirOperator::setupActions()
     QAction *upAction = d->actionCollection->addAction(KStandardAction::Up, QStringLiteral("up"), this, SLOT(cdUp()));
     upAction->setText(i18n("Parent Folder"));
 
-    d->actionCollection->addAction(KStandardAction::Back, QStringLiteral("back"), this, SLOT(back()));
+    QAction *backAction = d->actionCollection->addAction(KStandardAction::Back, QStringLiteral("back"), this, SLOT(back()));
+    backAction->setShortcut(Qt::Key_Backspace);
 
     d->actionCollection->addAction(KStandardAction::Forward, QStringLiteral("forward"), this, SLOT(forward()));
 
@@ -1976,9 +1987,22 @@ void KDirOperator::setupActions()
     d->actionCollection->addAction(QStringLiteral("by type"), byTypeAction);
     connect(byTypeAction, SIGNAL(triggered(bool)), this, SLOT(_k_slotSortByType()));
 
+    QActionGroup *sortOrderGroup = new QActionGroup(this);
+    sortOrderGroup->setExclusive(true);
+
+    KToggleAction *ascendingAction = new KToggleAction(i18n("Ascending"), this);
+    d->actionCollection->addAction(QStringLiteral("ascending"), ascendingAction);
+    ascendingAction->setActionGroup(sortOrderGroup);
+    connect(ascendingAction, &QAction::triggered, this, [this]() {
+        this->d->_k_slotSortReversed(false);
+    });
+
     KToggleAction *descendingAction = new KToggleAction(i18n("Descending"), this);
     d->actionCollection->addAction(QStringLiteral("descending"), descendingAction);
-    connect(descendingAction, SIGNAL(triggered(bool)), this, SLOT(_k_slotSortReversed(bool)));
+    descendingAction->setActionGroup(sortOrderGroup);
+    connect(descendingAction, &QAction::triggered, this, [this]() {
+        this->d->_k_slotSortReversed(true);
+    });
 
     KToggleAction *dirsFirstAction = new KToggleAction(i18n("Folders First"), this);
     d->actionCollection->addAction(QStringLiteral("dirs first"), dirsFirstAction);
@@ -2104,7 +2128,8 @@ void KDirOperator::setupActions()
     connect(d->newFileMenu, SIGNAL(directoryCreated(QUrl)), this, SLOT(_k_slotDirectoryCreated(QUrl)));
 
     d->actionCollection->addAssociatedWidget(this);
-    foreach (QAction *action, d->actionCollection->actions()) {
+    const QList<QAction *> list = d->actionCollection->actions();
+    for (QAction *action : list) {
         action->setShortcutContext(Qt::WidgetWithChildrenShortcut);
     }
 }
@@ -2124,7 +2149,9 @@ void KDirOperator::setupMenu(int whichActions)
     sortMenu->addAction(d->actionCollection->action(QStringLiteral("by date")));
     sortMenu->addAction(d->actionCollection->action(QStringLiteral("by type")));
     sortMenu->addSeparator();
+    sortMenu->addAction(d->actionCollection->action(QStringLiteral("ascending")));
     sortMenu->addAction(d->actionCollection->action(QStringLiteral("descending")));
+    sortMenu->addSeparator();
     sortMenu->addAction(d->actionCollection->action(QStringLiteral("dirs first")));
 
     // now plug everything into the popupmenu
@@ -2173,16 +2200,28 @@ void KDirOperator::setupMenu(int whichActions)
 
 void KDirOperator::updateSortActions()
 {
+    QAction *ascending = d->actionCollection->action(QStringLiteral("ascending"));
+    QAction *descending = d->actionCollection->action(QStringLiteral("descending"));
+
     if (KFile::isSortByName(d->sorting)) {
         d->actionCollection->action(QStringLiteral("by name"))->setChecked(true);
+        descending->setText(i18nc("Sort descending", "Z-A"));
+        ascending->setText(i18nc("Sort ascending", "A-Z"));
     } else if (KFile::isSortByDate(d->sorting)) {
         d->actionCollection->action(QStringLiteral("by date"))->setChecked(true);
+        descending->setText(i18nc("Sort descending", "Newest First"));
+        ascending->setText(i18nc("Sort ascending", "Oldest First"));
     } else if (KFile::isSortBySize(d->sorting)) {
         d->actionCollection->action(QStringLiteral("by size"))->setChecked(true);
+        descending->setText(i18nc("Sort descending", "Largest First"));
+        ascending->setText(i18nc("Sort ascending", "Smallest First"));
     } else if (KFile::isSortByType(d->sorting)) {
         d->actionCollection->action(QStringLiteral("by type"))->setChecked(true);
+        descending->setText(i18nc("Sort descending", "Z-A"));
+        ascending->setText(i18nc("Sort ascending", "A-Z"));
     }
-    d->actionCollection->action(QStringLiteral("descending"))->setChecked(d->sorting & QDir::Reversed);
+    ascending->setChecked(!(d->sorting & QDir::Reversed));
+    descending->setChecked(d->sorting & QDir::Reversed);
     d->actionCollection->action(QStringLiteral("dirs first"))->setChecked(d->sorting & QDir::DirsFirst);
 }
 
@@ -2370,6 +2409,11 @@ void KDirOperator::resizeEvent(QResizeEvent *)
     }
 }
 
+void KDirOperator::keyPressEvent(QKeyEvent *e) // TODO KF6 remove
+{
+    QWidget::keyPressEvent(e);
+}
+
 void KDirOperator::setOnlyDoubleClickSelectsFiles(bool enable)
 {
     d->onlyDoubleClickSelectsFiles = enable;
@@ -2384,6 +2428,26 @@ bool KDirOperator::onlyDoubleClickSelectsFiles() const
     return d->onlyDoubleClickSelectsFiles;
 }
 
+void KDirOperator::setFollowNewDirectories(bool enable)
+{
+    d->followNewDirectories = enable;
+}
+
+bool KDirOperator::followNewDirectories() const
+{
+    return d->followNewDirectories;
+}
+
+void KDirOperator::setFollowSelectedDirectories(bool enable)
+{
+    d->followSelectedDirectories = enable;
+}
+
+bool KDirOperator::followSelectedDirectories() const
+{
+    return d->followSelectedDirectories;
+}
+
 void KDirOperator::Private::_k_slotStarted()
 {
     progressBar->setValue(0);
@@ -2396,16 +2460,11 @@ void KDirOperator::Private::_k_slotShowProgress()
 {
     progressBar->raise();
     progressBar->show();
-    QApplication::flush();
 }
 
 void KDirOperator::Private::_k_slotProgress(int percent)
 {
     progressBar->setValue(percent);
-    // we have to redraw this as fast as possible
-    if (progressBar->isVisible()) {
-        QApplication::flush();
-    }
 }
 
 void KDirOperator::Private::_k_slotIOFinished()
@@ -2488,7 +2547,15 @@ void KDirOperator::Private::_k_slotActivated(const QModelIndex &index)
     }
 
     if (item.isDir()) {
-        parent->selectDir(item);
+        // Only allow disabling following selected directories on Tree and
+        // DetailTree views as selected directories in these views still expand
+        // when selected. For other views, disabling following selected
+        // directories would make selecting a directory a noop which is
+        // unintuitive.
+        if (followSelectedDirectories ||
+            (viewKind != KFile::Tree && viewKind != KFile::DetailTree)) {
+            parent->selectDir(item);
+        }
     } else {
         parent->selectFile(item);
     }
@@ -2753,7 +2820,9 @@ bool KDirOperator::Private::isReadable(const QUrl &url)
 
 void KDirOperator::Private::_k_slotDirectoryCreated(const QUrl &url)
 {
-    parent->setUrl(url, true);
+    if (followNewDirectories) {
+        parent->setUrl(url, true);
+    }
 }
 
 void KDirOperator::setSupportedSchemes(const QStringList &schemes)
