@@ -17,10 +17,11 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include "testtrash.h"
+
 #include <qtest.h>
 
 #include "kio_trash.h"
-#include "testtrash.h"
 #include "../../../pathhelpers_p.h"
 
 #include <kprotocolinfo.h>
@@ -86,8 +87,8 @@ QString TestTrash::otherTmpDir() const
 {
     // This one needs to be on another partition for the test to be meaningful
     QString tempDir = m_tempDir.path();
-    if (!tempDir.endsWith('/')) {
-        tempDir.append('/');
+    if (!tempDir.endsWith(QLatin1Char('/'))) {
+        tempDir.append(QLatin1Char('/'));
     }
     return tempDir;
 }
@@ -296,14 +297,14 @@ static void checkInfoFile(const QString &infoPath, const QString &origFilePath)
     const QString origPath = group.readEntry("Path");
     QVERIFY(!origPath.isEmpty());
     QCOMPARE(origPath.toUtf8(), QUrl::toPercentEncoding(origFilePath, "/"));
-    if (origFilePath.contains(QChar(0x2153)) || origFilePath.contains(QLatin1Char('%')) || origFilePath.contains(QStringLiteral("umlaut"))) {
+    if (origFilePath.contains(QChar(0x2153)) || origFilePath.contains(QLatin1Char('%')) || origFilePath.contains(QLatin1String("umlaut"))) {
         QVERIFY(origPath.contains(QLatin1Char('%')));
     } else {
         QVERIFY(!origPath.contains(QLatin1Char('%')));
     }
     const QString date = group.readEntry("DeletionDate");
     QVERIFY(!date.isEmpty());
-    QVERIFY(date.contains(QStringLiteral("T")));
+    QVERIFY(date.contains(QLatin1String("T")));
 }
 
 static void createTestFile(const QString &path)
@@ -351,7 +352,6 @@ void TestTrash::trashFile(const QString &origFilePath, const QString &fileId)
     QMap<QString, QString>::ConstIterator it = metaData.constBegin();
     for (; it != metaData.constEnd(); ++it) {
         if (it.key().startsWith(QLatin1String("trashURL"))) {
-            const QString origPath = it.key().mid(9);
             QUrl trashURL(it.value());
             qDebug() << trashURL;
             QVERIFY(!trashURL.isEmpty());
@@ -418,7 +418,7 @@ void TestTrash::trashFileIntoOtherPartition()
     }
     const QString fileName = QStringLiteral("testtrash-file");
     const QString origFilePath = m_otherPartitionTopDir + fileName;
-    const QString fileId = fileName;
+    const QString &fileId = fileName;
     // cleanup
     QFile::remove(m_otherPartitionTrashDir + QLatin1String("/info/") + fileId + QLatin1String(".trashinfo"));
     QFile::remove(m_otherPartitionTrashDir + QLatin1String("/files/") + fileId);
@@ -449,7 +449,6 @@ void TestTrash::trashFileIntoOtherPartition()
     QMap<QString, QString>::ConstIterator it = metaData.constBegin();
     for (; it != metaData.constEnd(); ++it) {
         if (it.key().startsWith(QLatin1String("trashURL"))) {
-            const QString origPath = it.key().mid(9);
             QUrl trashURL(it.value());
             qDebug() << trashURL;
             QVERIFY(!trashURL.isEmpty());
@@ -474,7 +473,6 @@ void TestTrash::trashFileOwnedByRoot()
     job->setUiDelegate(nullptr); // no skip dialog, thanks
     bool ok = job->exec();
     QVERIFY(!ok);
-    QMap<QString, QString> metaData = job->metaData();
 
     QVERIFY(job->error() == KIO::ERR_ACCESS_DENIED);
     const QString infoPath(m_trashDir + QLatin1String("/info/") + fileId + QLatin1String(".trashinfo"));
@@ -490,7 +488,7 @@ void TestTrash::trashSymlink(const QString &origFilePath, const QString &fileId,
 {
     // setup
     const char *target = broken ? "/nonexistent" : "/tmp";
-    bool ok = ::symlink(target, QFile::encodeName(origFilePath)) == 0;
+    bool ok = ::symlink(target, QFile::encodeName(origFilePath).constData()) == 0;
     QVERIFY(ok);
     QUrl u = QUrl::fromLocalFile(origFilePath);
 
@@ -506,7 +504,7 @@ void TestTrash::trashSymlink(const QString &origFilePath, const QString &fileId,
 
     QFileInfo files(m_trashDir + QLatin1String("/files/") + fileId);
     QVERIFY(files.isSymLink());
-    QVERIFY(files.readLink() == QFile::decodeName(target));
+    QVERIFY(files.symLinkTarget() == QFile::decodeName(target));
     QVERIFY(!QFile::exists(origFilePath));
 }
 
@@ -532,7 +530,7 @@ void TestTrash::trashDirectory(const QString &origPath, const QString &fileId)
 {
     qDebug() << fileId;
     // setup
-    if (!QFileInfo(origPath).exists()) {
+    if (!QFileInfo::exists(origPath)) {
         QDir dir;
         bool ok = dir.mkdir(origPath);
         QVERIFY(ok);
@@ -625,7 +623,7 @@ void TestTrash::trashReadOnlyDirFromHome()
     ok = dir.mkdir(subDirPath);
     QVERIFY(ok);
     createTestFile(subDirPath + QLatin1String("/testfile_in_subdir"));
-    ::chmod(QFile::encodeName(subDirPath), 0500);
+    ::chmod(QFile::encodeName(subDirPath).constData(), 0500);
 
     trashDirectory(dirName, QStringLiteral("readonly"));
 }
@@ -642,6 +640,21 @@ void TestTrash::trashDirectoryWithTrailingSlash()
     trashDirectory(homeTmpDir() + dirName, QStringLiteral("dirwithslash"));
 }
 
+void TestTrash::trashBrokenSymlinkIntoSubdir()
+{
+    QString origPath = homeTmpDir() + QStringLiteral("subDirBrokenSymlink");
+
+    if (!QFileInfo::exists(origPath)) {
+        QDir dir;
+        bool ok = dir.mkdir(origPath);
+        QVERIFY(ok);
+    }
+    bool ok = ::symlink("/nonexistent", QFile::encodeName(origPath + QStringLiteral("/link")).constData()) == 0;
+    QVERIFY(ok);
+
+    trashDirectory(origPath, QStringLiteral("subDirBrokenSymlink"));
+}
+
 void TestTrash::delRootFile()
 {
     // test deleting a trashed file
@@ -785,6 +798,20 @@ void TestTrash::statFileInDirectory()
     QCOMPARE(item.text(), QStringLiteral("testfile"));
 }
 
+void TestTrash::statBrokenSymlinkInSubdir()
+{
+    QUrl url(QStringLiteral("trash:/0-subDirBrokenSymlink/link"));
+    KIO::UDSEntry entry;
+    bool ok = MyNetAccess_stat(url, entry);
+    QVERIFY(ok);
+    KFileItem item(entry, url);
+    QVERIFY(item.isLink());
+    QVERIFY(item.isReadable());
+    QVERIFY(!item.isWritable());
+    QVERIFY(!item.isHidden());
+    QCOMPARE(item.linkDest(), QLatin1String("/nonexistent"));
+}
+
 void TestTrash::copyFromTrash(const QString &fileId, const QString &destPath, const QString &relativePath)
 {
     QUrl src(QLatin1String("trash:/0-") + fileId);
@@ -835,8 +862,8 @@ void TestTrash::copyDirectoryFromTrash()
     const QString destPath = otherTmpDir() + QLatin1String("trashDirFromHome_copied");
     copyFromTrash(QStringLiteral("trashDirFromHome"), destPath);
     QVERIFY(QFileInfo(destPath).isDir());
-    QVERIFY(QFile::exists(destPath + "/testfile"));
-    QVERIFY(QFile::exists(destPath + "/subdir/subfile"));
+    QVERIFY(QFile::exists(destPath + QStringLiteral("/testfile")));
+    QVERIFY(QFile::exists(destPath + QStringLiteral("/subdir/subfile")));
 }
 
 void TestTrash::copySymlinkFromTrash() // relies on trashSymlinkFromHome() being called first
@@ -859,15 +886,15 @@ void TestTrash::moveInTrash(const QString &fileId, const QString &destFileId)
     QVERIFY2(ok, qPrintable(job->errorString()));
 
     // Check old doesn't exist anymore
-    QString infoFile(m_trashDir + "/info/" + fileId + ".trashinfo");
+    QString infoFile(m_trashDir + QStringLiteral("/info/") + fileId + QStringLiteral(".trashinfo"));
     QVERIFY(!QFile::exists(infoFile));
-    QFileInfo filesItem(m_trashDir + "/files/" + fileId);
+    QFileInfo filesItem(m_trashDir + QStringLiteral("/files/") + fileId);
     QVERIFY(!filesItem.exists());
 
     // Check new exists now
-    QString newInfoFile(m_trashDir + "/info/" + destFileId + ".trashinfo");
+    QString newInfoFile(m_trashDir + QStringLiteral("/info/") + destFileId + QStringLiteral(".trashinfo"));
     QVERIFY(QFile::exists(newInfoFile));
-    QFileInfo newFilesItem(m_trashDir + "/files/" + destFileId);
+    QFileInfo newFilesItem(m_trashDir + QStringLiteral("/files/") + destFileId);
     QVERIFY(newFilesItem.exists());
 
 }
@@ -910,10 +937,10 @@ void TestTrash::moveFromTrash(const QString &fileId, const QString &destPath, co
     KIO::Job *job = KIO::moveAs(src, dest, KIO::HideProgressInfo);
     bool ok = job->exec();
     QVERIFY2(ok, qPrintable(job->errorString()));
-    QString infoFile(m_trashDir + "/info/" + fileId + ".trashinfo");
+    QString infoFile(m_trashDir + QStringLiteral("/info/") + fileId + QStringLiteral(".trashinfo"));
     QVERIFY(!QFile::exists(infoFile));
 
-    QFileInfo filesItem(m_trashDir + "/files/" + fileId);
+    QFileInfo filesItem(m_trashDir + QStringLiteral("/files/") + fileId);
     QVERIFY(!filesItem.exists());
 
     QVERIFY(QFile::exists(destPath));
@@ -928,7 +955,7 @@ void TestTrash::moveFileFromTrash()
     const QFile::Permissions origPerms = QFileInfo(filePath).permissions();
     trashFile(filePath, fileName);
 
-    const QString destPath = otherTmpDir() + "fileFromTrash_restored";
+    const QString destPath = otherTmpDir() + QStringLiteral("fileFromTrash_restored");
     moveFromTrash(fileName, destPath);
     const QFileInfo destInfo(destPath);
     QVERIFY(destInfo.isFile());
@@ -960,7 +987,7 @@ void TestTrash::moveFileFromTrashToDir()
 
     // When moving it out to a dir
     QFETCH(QString, destDir);
-    const QString destPath = destDir + "moveFileFromTrashToDir";
+    const QString destPath = destDir + QStringLiteral("moveFileFromTrashToDir");
     const QUrl src(QLatin1String("trash:/0-") + fileName);
     const QUrl dest(QUrl::fromLocalFile(destDir));
     KIO::Job *job = KIO::move(src, dest, KIO::HideProgressInfo);
@@ -979,7 +1006,7 @@ void TestTrash::moveFileFromTrashToDir()
 
 void TestTrash::moveFileInDirectoryFromTrash()
 {
-    const QString destPath = otherTmpDir() + "testfile_restored";
+    const QString destPath = otherTmpDir() + QStringLiteral("testfile_restored");
     copyFromTrash(QStringLiteral("trashDirFromHome"), destPath, QStringLiteral("testfile"));
     QVERIFY(QFileInfo(destPath).isFile());
     QVERIFY(QFileInfo(destPath).size() == 12);
@@ -987,7 +1014,7 @@ void TestTrash::moveFileInDirectoryFromTrash()
 
 void TestTrash::moveDirectoryFromTrash()
 {
-    const QString destPath = otherTmpDir() + "trashDirFromHome_restored";
+    const QString destPath = otherTmpDir() + QStringLiteral("trashDirFromHome_restored");
     moveFromTrash(QStringLiteral("trashDirFromHome"), destPath);
     QVERIFY(QFileInfo(destPath).isDir());
     checkDirCacheValidity();
@@ -1018,15 +1045,14 @@ void TestTrash::trashDirectoryOwnedByRoot()
     job->setUiDelegate(nullptr); // no skip dialog, thanks
     bool ok = job->exec();
     QVERIFY(!ok);
-    QMap<QString, QString> metaData = job->metaData();
     const int err = job->error();
     QVERIFY(err == KIO::ERR_ACCESS_DENIED
             || err == KIO::ERR_CANNOT_OPEN_FOR_READING);
 
-    const QString infoPath(m_trashDir + "/info/" + fileId + ".trashinfo");
+    const QString infoPath(m_trashDir + QStringLiteral("/info/") + fileId + QStringLiteral(".trashinfo"));
     QVERIFY(!QFile::exists(infoPath));
 
-    QFileInfo files(m_trashDir + "/files/" + fileId);
+    QFileInfo files(m_trashDir + QStringLiteral("/files/") + fileId);
     QVERIFY(!files.exists());
 
     QVERIFY(QFile::exists(u.path()));
@@ -1034,7 +1060,7 @@ void TestTrash::trashDirectoryOwnedByRoot()
 
 void TestTrash::moveSymlinkFromTrash()
 {
-    const QString destPath = otherTmpDir() + "symlinkFromHome_restored";
+    const QString destPath = otherTmpDir() + QStringLiteral("symlinkFromHome_restored");
     moveFromTrash(QStringLiteral("symlinkFromHome"), destPath);
     QVERIFY(QFileInfo(destPath).isSymLink());
 }
@@ -1064,8 +1090,8 @@ void TestTrash::restoreFile()
 {
     const QString fileId = QStringLiteral("fileFromHome (1)");
     const QUrl url = TrashImpl::makeURL(0, fileId, QString());
-    const QString infoFile(m_trashDir + "/info/" + fileId + ".trashinfo");
-    const QString filesItem(m_trashDir + "/files/" + fileId);
+    const QString infoFile(m_trashDir + QStringLiteral("/info/") + fileId + QStringLiteral(".trashinfo"));
+    const QString filesItem(m_trashDir + QStringLiteral("/files/") + fileId);
 
     QVERIFY(QFile::exists(infoFile));
     QVERIFY(QFile::exists(filesItem));
@@ -1080,18 +1106,18 @@ void TestTrash::restoreFile()
     QVERIFY(!QFile::exists(infoFile));
     QVERIFY(!QFile::exists(filesItem));
 
-    const QString destPath = homeTmpDir() + "fileFromHome";
+    const QString destPath = homeTmpDir() + QStringLiteral("fileFromHome");
     QVERIFY(QFile::exists(destPath));
 }
 
 void TestTrash::restoreFileFromSubDir()
 {
     const QString fileId = QStringLiteral("trashDirFromHome (1)/testfile");
-    QVERIFY(!QFile::exists(homeTmpDir() + "trashDirFromHome (1)"));
+    QVERIFY(!QFile::exists(homeTmpDir() + QStringLiteral("trashDirFromHome (1)")));
 
     const QUrl url = TrashImpl::makeURL(0, fileId, QString());
-    const QString infoFile(m_trashDir + "/info/trashDirFromHome (1).trashinfo");
-    const QString filesItem(m_trashDir + "/files/trashDirFromHome (1)/testfile");
+    const QString infoFile(m_trashDir + QStringLiteral("/info/trashDirFromHome (1).trashinfo"));
+    const QString filesItem(m_trashDir + QStringLiteral("/files/trashDirFromHome (1)/testfile"));
 
     QVERIFY(QFile::exists(infoFile));
     QVERIFY(QFile::exists(filesItem));
@@ -1108,7 +1134,7 @@ void TestTrash::restoreFileFromSubDir()
     // check that nothing happened
     QVERIFY(QFile::exists(infoFile));
     QVERIFY(QFile::exists(filesItem));
-    QVERIFY(!QFile::exists(homeTmpDir() + "trashDirFromHome (1)"));
+    QVERIFY(!QFile::exists(homeTmpDir() + QStringLiteral("trashDirFromHome (1)")));
 }
 
 void TestTrash::restoreFileToDeletedDirectory()
@@ -1124,8 +1150,8 @@ void TestTrash::restoreFileToDeletedDirectory()
 
     const QString fileId = QStringLiteral("fileFromHome");
     const QUrl url = TrashImpl::makeURL(0, fileId, QString());
-    const QString infoFile(m_trashDir + "/info/" + fileId + ".trashinfo");
-    const QString filesItem(m_trashDir + "/files/" + fileId);
+    const QString infoFile(m_trashDir + QStringLiteral("/info/") + fileId + QStringLiteral(".trashinfo"));
+    const QString filesItem(m_trashDir + QStringLiteral("/files/") + fileId);
 
     QVERIFY(QFile::exists(infoFile));
     QVERIFY(QFile::exists(filesItem));
@@ -1143,7 +1169,7 @@ void TestTrash::restoreFileToDeletedDirectory()
     QVERIFY(QFile::exists(infoFile));
     QVERIFY(QFile::exists(filesItem));
 
-    const QString destPath = homeTmpDir() + "fileFromHome";
+    const QString destPath = homeTmpDir() + QStringLiteral("fileFromHome");
     QVERIFY(!QFile::exists(destPath));
 }
 
@@ -1153,8 +1179,8 @@ void TestTrash::listRootDir()
     m_listResult.clear();
     m_displayNameListResult.clear();
     KIO::ListJob *job = KIO::listDir(QUrl(QStringLiteral("trash:/")), KIO::HideProgressInfo);
-    connect(job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
-            SLOT(slotEntries(KIO::Job*,KIO::UDSEntryList)));
+    connect(job, &KIO::ListJob::entries,
+            this, &TestTrash::slotEntries);
     bool ok = job->exec();
     QVERIFY(ok);
     qDebug() << "listDir done - m_entryCount=" << m_entryCount;
@@ -1173,8 +1199,8 @@ void TestTrash::listRecursiveRootDir()
     m_listResult.clear();
     m_displayNameListResult.clear();
     KIO::ListJob *job = KIO::listRecursive(QUrl(QStringLiteral("trash:/")), KIO::HideProgressInfo);
-    connect(job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
-            SLOT(slotEntries(KIO::Job*,KIO::UDSEntryList)));
+    connect(job, &KIO::ListJob::entries,
+            this, &TestTrash::slotEntries);
     bool ok = job->exec();
     QVERIFY(ok);
     qDebug() << "listDir done - m_entryCount=" << m_entryCount;
@@ -1187,11 +1213,12 @@ void TestTrash::listRecursiveRootDir()
     QCOMPARE(m_listResult.count(QStringLiteral("0-fileFromHome (1)")), 1);
     QCOMPARE(m_listResult.count(QStringLiteral("0-trashDirFromHome/testfile")), 1);
     QCOMPARE(m_listResult.count(QStringLiteral("0-readonly/readonly_subdir/testfile_in_subdir")), 1);
+    QCOMPARE(m_listResult.count(QStringLiteral("0-subDirBrokenSymlink/link")), 1);
     QCOMPARE(m_displayNameListResult.count(QStringLiteral("fileFromHome")), 1);
     QCOMPARE(m_displayNameListResult.count(QStringLiteral("fileFromHome (1)")), 1);
     QCOMPARE(m_displayNameListResult.count(QStringLiteral("trashDirFromHome/testfile")), 1);
     QCOMPARE(m_displayNameListResult.count(QStringLiteral("readonly/readonly_subdir/testfile_in_subdir")), 1);
-
+    QCOMPARE(m_displayNameListResult.count(QStringLiteral("subDirBrokenSymlink/link")), 1);
 }
 
 void TestTrash::listSubDir()
@@ -1200,8 +1227,8 @@ void TestTrash::listSubDir()
     m_listResult.clear();
     m_displayNameListResult.clear();
     KIO::ListJob *job = KIO::listDir(QUrl(QStringLiteral("trash:/0-trashDirFromHome")), KIO::HideProgressInfo);
-    connect(job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)),
-            SLOT(slotEntries(KIO::Job*,KIO::UDSEntryList)));
+    connect(job, &KIO::ListJob::entries,
+            this, &TestTrash::slotEntries);
     bool ok = job->exec();
     QVERIFY(ok);
     qDebug() << "listDir done - m_entryCount=" << m_entryCount;
@@ -1218,8 +1245,7 @@ void TestTrash::listSubDir()
 
 void TestTrash::slotEntries(KIO::Job *, const KIO::UDSEntryList &lst)
 {
-    for (KIO::UDSEntryList::ConstIterator it = lst.begin(); it != lst.end(); ++it) {
-        const KIO::UDSEntry &entry(*it);
+    for (const KIO::UDSEntry &entry : lst) {
         QString name = entry.stringValue(KIO::UDSEntry::UDS_NAME);
         QString displayName = entry.stringValue(KIO::UDSEntry::UDS_DISPLAY_NAME);
         QUrl url(entry.stringValue(KIO::UDSEntry::UDS_URL));
