@@ -22,12 +22,10 @@
 #include "ksslinfodialog.h"
 #include "ui_sslinfo.h"
 #include "ksslcertificatebox.h"
+#include "ksslerror_p.h"
 
-#include <QProcess>
 
 #include <QDialogButtonBox>
-#include <QLabel>
-#include <QLayout>
 
 #include <QSslCertificate>
 
@@ -40,7 +38,7 @@ class Q_DECL_HIDDEN KSslInfoDialog::KSslInfoDialogPrivate
 {
 public:
     QList<QSslCertificate> certificateChain;
-    QList<QList<KSslError::Error> > certificateErrors;
+    QList<QList<QSslError::SslError>> certificateErrors;
 
     bool isMainPartEncrypted;
     bool auxPartsEncrypted;
@@ -116,20 +114,24 @@ void KSslInfoDialog::updateWhichPartsEncrypted()
 {
     if (d->isMainPartEncrypted) {
         if (d->auxPartsEncrypted) {
-            d->ui.encryptionIndicator->setPixmap(BarIcon(QStringLiteral("security-high")));
+            d->ui.encryptionIndicator->setPixmap(QIcon::fromTheme(QStringLiteral("security-high"))
+                .pixmap(KIconLoader::SizeSmallMedium));
             d->ui.explanation->setText(i18n("Current connection is secured with SSL."));
         } else {
-            d->ui.encryptionIndicator->setPixmap(BarIcon(QStringLiteral("security-medium")));
+            d->ui.encryptionIndicator->setPixmap(QIcon::fromTheme(QStringLiteral("security-medium"))
+                .pixmap(KIconLoader::SizeSmallMedium));
             d->ui.explanation->setText(i18n("The main part of this document is secured "
                                             "with SSL, but some parts are not."));
         }
     } else {
         if (d->auxPartsEncrypted) {
-            d->ui.encryptionIndicator->setPixmap(BarIcon(QStringLiteral("security-medium")));
+            d->ui.encryptionIndicator->setPixmap(QIcon::fromTheme(QStringLiteral("security-medium"))
+                .pixmap(KIconLoader::SizeSmallMedium));
             d->ui.explanation->setText(i18n("Some of this document is secured with SSL, "
                                             "but the main part is not."));
         } else {
-            d->ui.encryptionIndicator->setPixmap(BarIcon(QStringLiteral("security-low")));
+            d->ui.encryptionIndicator->setPixmap(QIcon::fromTheme(QStringLiteral("security-low"))
+                .pixmap(KIconLoader::SizeSmallMedium));
             d->ui.explanation->setText(i18n("Current connection is not secured with SSL."));
         }
     }
@@ -141,13 +143,31 @@ void KSslInfoDialog::setSslInfo(const QList<QSslCertificate> &certificateChain,
                                 int usedBits, int bits,
                                 const QList<QList<KSslError::Error> > &validationErrors)
 {
+    QList<QList<QSslError::SslError>> qValidationErrors;
+    qValidationErrors.reserve(validationErrors.size());
+    for (const auto &l : validationErrors) {
+        QList<QSslError::SslError> qErrors;
+        qErrors.reserve(l.size());
+        for (const KSslError::Error e : l) {
+            qErrors.push_back(KSslErrorPrivate::errorFromKSslError(e));
+        }
+        qValidationErrors.push_back(qErrors);
+    }
+    setSslInfo(certificateChain, ip, host, sslProtocol, cipher, usedBits, bits, qValidationErrors);
+}
+
+void KSslInfoDialog::setSslInfo(const QList<QSslCertificate> &certificateChain,
+                                const QString &ip, const QString &host,
+                                const QString &sslProtocol, const QString &cipher,
+                                int usedBits, int bits,
+                                const QList<QList<QSslError::SslError>> &validationErrors)
+{
 
     d->certificateChain = certificateChain;
     d->certificateErrors = validationErrors;
 
     d->ui.certSelector->clear();
-    for (int i = 0; i < certificateChain.size(); i++) {
-        const QSslCertificate &cert = certificateChain[i];
+    for (const QSslCertificate &cert : certificateChain) {
         QString name;
         static const QSslCertificate::SubjectInfo si[] = {
             QSslCertificate::CommonName,
@@ -156,7 +176,7 @@ void KSslInfoDialog::setSslInfo(const QList<QSslCertificate> &certificateChain,
         };
         for (int j = 0; j < 3 && name.isEmpty(); j++)
         {
-            name = cert.subjectInfo(si[j]).join(QStringLiteral(", "));
+            name = cert.subjectInfo(si[j]).join(QLatin1String(", "));
         }
         d->ui.certSelector->addItem(name);
     }
@@ -195,10 +215,11 @@ void KSslInfoDialog::displayFromChain(int i)
     const QSslCertificate &cert = d->certificateChain[i];
 
     QString trusted;
-    if (!d->certificateErrors[i].isEmpty()) {
+    const QList<QSslError::SslError> errorsList = d->certificateErrors[i];
+    if (!errorsList.isEmpty()) {
         trusted = i18nc("The certificate is not trusted", "NO, there were errors:");
-        foreach (KSslError::Error e, d->certificateErrors[i]) {
-            KSslError classError(e);
+        for (QSslError::SslError e : errorsList) {
+            QSslError classError(e);
             trusted += QLatin1Char('\n') + classError.errorString();
         }
     } else {
@@ -230,7 +251,7 @@ QList<QList<KSslError::Error> > KSslInfoDialog::errorsFromString(const QString &
         const QStringList sl2 = s.split(QLatin1Char('\t'), QString::SkipEmptyParts);
         for (const QString &s2 : sl2) {
             bool didConvert;
-            KSslError::Error error = static_cast<KSslError::Error>(s2.toInt(&didConvert));
+            KSslError::Error error = KSslErrorPrivate::errorFromQSslError(static_cast<QSslError::SslError>(s2.toInt(&didConvert)));
             if (didConvert) {
                 certErrors.append(error);
             }
@@ -240,3 +261,23 @@ QList<QList<KSslError::Error> > KSslInfoDialog::errorsFromString(const QString &
     return ret;
 }
 
+//static
+QList<QList<QSslError::SslError>> KSslInfoDialog::certificateErrorsFromString(const QString &errorsString)
+{
+    const QStringList sl = errorsString.split(QLatin1Char('\n'), QString::KeepEmptyParts);
+    QList<QList<QSslError::SslError>> ret;
+    ret.reserve(sl.size());
+    for (const QString &s : sl) {
+        QList<QSslError::SslError> certErrors;
+        const QStringList sl2 = s.split(QLatin1Char('\t'), QString::SkipEmptyParts);
+        for (const QString &s2 : sl2) {
+            bool didConvert;
+            QSslError::SslError error = static_cast<QSslError::SslError>(s2.toInt(&didConvert));
+            if (didConvert) {
+                certErrors.append(error);
+            }
+        }
+        ret.append(certErrors);
+    }
+    return ret;
+}
